import math


class QuadNode:
  def __init__(self,minX,maxX,minY,maxY,res):
    self.minX = minX
    self.maxX = maxX
    self.minY = minY
    self.maxY = maxY

    self.split = ((maxX-minX)>res) or ((maxY-minY)>res)

    if self.split:
      self.splitX = 0.5*(minX + maxX)
      self.splitY = 0.5*(minY + maxY)

      self.s = ((QuadNode(self.minX,self.splitX,self.minY,self.splitY,res),
                 QuadNode(self.splitX,self.maxX,self.minY,self.splitY,res)),
                (QuadNode(self.minX,self.splitX,self.splitY,self.maxY,res),
                 QuadNode(self.splitX,self.maxX,self.splitY,self.maxY,res))) # [y<split][x<split]
    else:
      self.data = dict() # Indexed by id() of stored object.

  def add(self,obj,x,y):
    if self.split:
      return self.s[int(y>=self.splitY)][int(x>=self.splitX)].add(obj,x,y)
    else:
      self.data[id(obj)] = [obj,x,y]
      return self

  def inRange(self,minX,maxX,minY,maxY):
    """Generates all objects that *could* be inside the given range - further pruning will usually be needed."""
    if self.split:
      valid = []
      
      lx = True
      hx = True
      ly = True
      hy = True

      if minX>=self.splitX: lx = False
      elif maxX<self.splitX: hx = False
      if minY>=self.splitY: ly = False
      elif maxY<self.splitY: hy = False

      if lx and ly: valid.append(self.s[0][0])
      if hx and ly: valid.append(self.s[0][1])
      if lx and hy: valid.append(self.s[1][0])
      if hx and hy: valid.append(self.s[1][1])
    
      for v in valid:
        for x in v.inRange(minX,maxX,minY,maxY):
          yield x
    else:
      for k,obj in self.data.iteritems():
        yield obj


class QuadTree:
  def __init__(self,minX,maxX,minY,maxY,res = 8.0):
    self.tree = QuadNode(minX,maxX,minY,maxY,res)

    self.byId = dict() # Converts an object from id() to the leaf Quad node its in.


  def update(self,obj,x,y):
    """Given an object and a position updates it in the database. If it doesn't exist its added, otherwise the current record is updated to its new position, if needed."""
    oKey = id(obj)
    if self.byId.has_key(oKey):
      node = self.byId[oKey]
      if x<node.minX or x>=node.maxX or y<node.minY or y>=node.maxY:
        # Its changed leaf - update...
        del node.data[oKey]
        self.byId[oKey] = self.tree.add(obj,x,y)
      else:
        r = node.data[oKey]
        r[1] = x
        r[2] = y
    else:
      self.byId[oKey] = self.tree.add(obj,x,y)

  def remove(self,obj):
    oKey = id(obj)
    if self.byId.has_key(oKey):
      node = self.byId[oKey]
      del node.data[oKey]
      del self.byId[oKey]

  def inCircle(self,x,y,radius):
    """Generates all objects that are within the given circle."""
    for obj,ox,oy in self.tree.inRange(x-radius,x+radius,y-radius,y+radius):
      if math.sqrt((ox-x)**2.0 + (oy-y)**2.0) < radius:
        yield obj

  def inPolygon(self,poly,border):
    """Generates all objects that are within the given polygon; supports a border around the edge to grow the polygon a little."""
    # First generate the relevant test set, with repeat calculations dealt with; also calculate the bounding volume...
    minX = poly[0][0]
    maxX = poly[0][0]
    minY = poly[0][1]
    maxY = poly[0][1]

    tests = [] # Each test is a base point, a perpendicular normal and a direction of vector.
    for i in xrange(len(poly)):
      a = poly[i]
      b = poly[(i+1)%len(poly)]

      minX = min(minX,a[0])
      maxX = max(maxX,a[0])
      minY = min(minY,a[1])
      maxY = max(maxY,a[1])
      
      vec = (b[0]-a[0],b[1]-a[1])
      length = math.sqrt(vec[0]**2.0 + vec[1]**2.0)
      vec = (vec[0]/length,vec[1]/length)
      perp = (-vec[1],vec[0])
      
      tests.append((a,perp,vec,length))

    # Now get all candidates and test each one...
    for obj,ox,oy in self.tree.inRange(minX-border,maxX+border,minY-border,maxY+border):
      posCount = 0
      negCount = 0 # Note that we don't count when the object is within border of the edge.
      
      for test in tests:
        ax = ox - test[0][0]
        ay = oy - test[0][1]
        
        edgeDist = ax*test[1][0] + ay*test[1][1]

        withinBorder = False
        if abs(edgeDist)<border:
          edgeTan = ax*test[2][0] + ay*test[2][1]
          if edgeTan>-border and edgeTan<test[3]+border:
            withinBorder = True

        if not withinBorder:
          if edgeDist>0.0:
            posCount += 1
          else:
            negCount += 1

      if posCount==0 or negCount==0:
        yield obj
      
